// 005 u_c_state_20
// 11 dec 2004

// -- Delphi implementation from the book:
// --   Design Patterns Elements of Reusable Object-Oriented Software
// --   Addison Wesley - 1994 - ISBN 0-201-63361-2
// -- (C) Erich GAMMA, Richard HELM, Ralph JOHNSON, John VLISSIDES 1994

// -- translation from C to Delphi performed by
// --   Felix John COLIBRI  http://www.felix-colibri.com

(*$r+*)

unit u_c_state_20;
  interface

    type c_tcp_octet_stream= Class
                             end; // c_tcp_octet_stream
                             
         c_tcp_state= Class; // forward

         // -- abstract
         c_tcp_connection= class
                             // friend tcp_state
                             _m_c_tcp_state: c_tcp_state;

                             Constructor create_tcp_connection;
                             procedure active_open;
                             procedure passive_open;
                             procedure close;
                             procedure send;
                             procedure acknowledge;
                             procedure synchronize;

                             procedure process_octet(p_c_tcp_octet_stream: c_tcp_octet_stream);
                             procedure _change_tcp_state(p_c_tcp_state: c_tcp_state);
                             Destructor Destroy; Override;
                           end; // c_tcp_connection

         // -- abstract
         c_tcp_state= Class
                        constructor create_tcp_state;
                        procedure transmit(p_tcp_connection: c_tcp_connection; p_tcp_octet_stream: c_tcp_octet_stream); Virtual;
                        procedure active_open(p_tcp_connection: c_tcp_connection); Virtual;
                        procedure passive_open(p_tcp_connection: c_tcp_connection); Virtual;
                        procedure close(p_tcp_connection: c_tcp_connection); Virtual;
                        procedure send(p_tcp_connection: c_tcp_connection); Virtual;
                        procedure acknowledge(p_tcp_connection: c_tcp_connection); Virtual;
                        procedure synchronize(p_tcp_connection: c_tcp_connection); Virtual;

                        procedure _change_tcp_state(p_tcp_connection: c_tcp_connection; p_c_tcp_state: c_tcp_state); Virtual;

                        Destructor Destroy; Override;
                      end; // c_tcp_state

         // -- maintain no state (can be shared), are singletons
         c_tcp_established= class(c_tcp_state)
                              class function f_c_tcp_established_instance: c_tcp_state;
                              procedure transmit(p_c_tcp_connection: c_tcp_connection; p_c_tcp_octet_stream: c_tcp_octet_stream); Virtual;
                              procedure close(p_tcp_connection: c_tcp_connection); Override;
                            end; // c_tcp_established

         c_tcp_listen= class(c_tcp_state)
                         class function f_c_tcp_listen_instance: c_tcp_state;
                         procedure send(p_tcp_connection: c_tcp_connection); Override;
                       end; // c_tcp_listen

         c_tcp_closed= class(c_tcp_state)
                         class function f_c_tcp_closed_instance: c_tcp_state;
                         procedure active_open(p_tcp_connection: c_tcp_connection); Override;
                         procedure passive_open(p_tcp_connection: c_tcp_connection); Override;
                       end; // c_tcp_closed

  implementation

    // -- c_tcp_connection

    Constructor c_tcp_connection.create_tcp_connection;
      begin
        Inherited Create;
        // _m_c_tcp_state:= c_tcp_closed.f_c_state_instance;
      end; // create_tcp_connection

    procedure c_tcp_connection.active_open;
      begin
        _m_c_tcp_state.active_open(Self);
      end; // active_open

    procedure c_tcp_connection.passive_open;
      begin
        _m_c_tcp_state.passive_open(Self);
      end; // passive_open

    procedure c_tcp_connection.close;
      begin
        _m_c_tcp_state.close(Self);
      end; // close

    procedure c_tcp_connection.send;
      begin
      end; // send

    procedure c_tcp_connection.acknowledge;
      begin
        _m_c_tcp_state.acknowledge(Self);
      end; // acknowledge

    procedure c_tcp_connection.synchronize;
      begin
        _m_c_tcp_state.synchronize(Self);
      end; // synchronize

    procedure c_tcp_connection.process_octet(p_c_tcp_octet_stream: c_tcp_octet_stream);
      begin
      end; // process_octet

    procedure c_tcp_connection._change_tcp_state(p_c_tcp_state: c_tcp_state);
      begin
        _m_c_tcp_state:= p_c_tcp_state;
      end; // _change_tcp_state

    Destructor c_tcp_connection.Destroy;
      begin
        Inherited;
      end; // Destroy

    // -- c_tcp_state

    constructor c_tcp_state.create_tcp_state;
      begin
      end; // create_tcp_state

    procedure c_tcp_state.transmit(p_tcp_connection: c_tcp_connection; p_tcp_octet_stream: c_tcp_octet_stream); 
      begin
      end; // transmit

    procedure c_tcp_state.active_open(p_tcp_connection: c_tcp_connection);
      begin
      end; // active_open

    procedure c_tcp_state.passive_open(p_tcp_connection: c_tcp_connection);
      begin
      end; // passive_open

    procedure c_tcp_state.close(p_tcp_connection: c_tcp_connection);
      begin
      end; // close
      
    procedure c_tcp_state.send(p_tcp_connection: c_tcp_connection);
      begin
      end; // send

    procedure c_tcp_state.acknowledge(p_tcp_connection: c_tcp_connection);
      begin
      end; // acknowledge

    procedure c_tcp_state.synchronize(p_tcp_connection: c_tcp_connection); 
      begin
      end; // synchronize

    procedure c_tcp_state._change_tcp_state(p_tcp_connection: c_tcp_connection; p_c_tcp_state: c_tcp_state);
      begin
      end; // _change_tcp_state

    Destructor c_tcp_state.Destroy;
      begin
      end; // Destroy

    // -- c_tcp_established

    var _g_c_tcp_established_instance: c_tcp_established= Nil;

    class function c_tcp_established.f_c_tcp_established_instance: c_tcp_state;
      begin
        if _g_c_tcp_established_instance= Nil
          then _g_c_tcp_established_instance:= c_tcp_established.create_tcp_state;
        Result:= _g_c_tcp_established_instance;
      end; // f_c_tcp_established_instance

    procedure c_tcp_established.transmit(p_c_tcp_connection: c_tcp_connection; p_c_tcp_octet_stream: c_tcp_octet_stream);
      begin
        p_c_tcp_connection.process_octet(p_c_tcp_octet_stream);
      end; // transmit

    procedure c_tcp_established.close(p_tcp_connection: c_tcp_connection);
      begin
        // -- send FIN, receive ACK, FIN

        _change_tcp_state(p_tcp_connection, c_tcp_listen.f_c_tcp_listen_instance);
      end; // close

    // -- c_tcp_listen

    var _g_c_tcp_listen_instance: c_tcp_listen= Nil;

    class function c_tcp_listen.f_c_tcp_listen_instance: c_tcp_state;
      begin
        if _g_c_tcp_listen_instance= Nil
          then _g_c_tcp_listen_instance:= c_tcp_listen.create_tcp_state;
        Result:= _g_c_tcp_listen_instance;
      end; // f_c_tcp_listen_instance

    procedure c_tcp_listen.send(p_tcp_connection: c_tcp_connection);
      begin
        // -- send SYN, receive SYN, ACK etc

        _change_tcp_state(p_tcp_connection, c_tcp_established.f_c_tcp_established_instance);
      end; // send

    // -- c_tcp_close

    var _g_c_tcp_closed_instance: c_tcp_closed= Nil;

    class function c_tcp_closed.f_c_tcp_closed_instance: c_tcp_state;
      begin
        if _g_c_tcp_closed_instance= Nil
          then _g_c_tcp_closed_instance:= c_tcp_closed.create_tcp_state;
        Result:= _g_c_tcp_closed_instance;
      end; // f_c_instance

    procedure c_tcp_closed.active_open(p_tcp_connection: c_tcp_connection);
      begin
        // -- sent SYN, receive SYN ACK etc

        _change_tcp_state(p_tcp_connection, c_tcp_established.f_c_tcp_established_instance);
      end; // active_open

    procedure c_tcp_closed.passive_open(p_tcp_connection: c_tcp_connection);
      begin
        _change_tcp_state(p_tcp_connection, c_tcp_listen.f_c_tcp_listen_instance);
      end; // passive_open







    begin // u_c_tcp_connection
    end. // u_c_tcp_connection
